//***************************************************************************************
//Program Diplomka
//
//
//
//
//***************************************************************************************

/* Knihovny */
#include <stdio.h>
#include <math.h>
#include <msp430.h>
#include "blink.h"

//**************************************************************
//**************************************************************
/* I2C Adresy */
/* SHT21 */
#define SadrSHTR		0x81
#define SadrSHTW		0x80

/* HT16K33 */
#define SadrHT16W		0xE0

/* MPR121 */
#define   Write_MPR121     0xB4
#define   Read_MPR121      0xB5

/* BMP180 */
#define   BMP180W     0xEE
#define   BMP180R     0xEF

/* MMA8652FC */
#define   MMA8652W     0x3A
#define   MMA8652R     0x3B

//**************************************************************
//**************************************************************
//**************************************************************
//**************************************************************
/* I/O */
//I2C
#define SDA_1()    (P1OUT |=  BIT3)	/* SDA Output=1 */
#define SDA_0()    (P1OUT &= ~BIT3)	/* SDA OUTPUT=0 */
#define SDA_INPUT  (P1DIR &= ~BIT3)	/* SDA INPUT */
#define SDA_OUTPUT (P1DIR |=  BIT3)	/* SDA OUTPUT */
#define SDA_IN     (P1IN  &   BIT3)
#define SCL_0()    (P1OUT &= ~BIT2)	/* SCL Output=0 */
#define SCL_1()    (P1OUT |=  BIT2)	/* SCL Output=1 */

//Piezo
#define PIEZO_1()    (P3OUT |=  0x10)	/* Piezo Output=1 */
#define PIEZO_0()    (P3OUT &=  ~0x10)	/* Piezo Output=0 */


//**************************************************************
//**************************************************************
//**************************************************************
//**************************************************************
/* Nazvy registru */

/* command SHT21  r/w */
#define  MEASURE_T		0xE3	/* 1110 0011 */
#define  MEASURE_RH		0xE5	/* 1110 0101 */
#define  WRITE_REG		0xE6	/* 1110 0110 */
#define  READ_REG		0xE7	/* 1110 0111 */
#define  SOFT_RESET		0xFE	/* 1111 1110 */

/* command HT16K33  W */
#define  SYSSET		0x21	/* 0010 XXX1 */
#define  ROWSET		0xA0	/* 1010 XXX0 */
#define  DISSET		0x80	/* 1000 X001 */
#define  DUTYSET	0xE7	/* 1110 1111 */

/* command MPR121 */
#define   ELE_L            0x00
#define   ELE_H            0x01
#define   OOR_STATU        0x02
#define   ELEPROX          0x03
#define   ELE0_L           0x04
#define   ELE0_H           0x05
#define   ELE1_L           0x06
#define   ELE1_H           0x07
#define   ELE2_L           0x08
#define   ELE2_H           0x09
#define   ELE3_L           0x0A
#define   ELE3_H           0x0B
#define   ELE4_L           0x0C
#define   ELE4_H           0x0D
#define   ELE5_L           0x0E
#define   ELE5_H           0x0F
#define   ELE6_L           0x10
#define   ELE6_H           0x11
#define   ELE7_L           0x12
#define   ELE7_H           0x13
#define   ELE8_L           0x14
#define   ELE8_H           0x15
#define   ELE9_L           0x16
#define   ELE9_H           0x17
#define   ELE10_L          0x18
#define   ELE10_H          0x19
#define   ELE11_L          0x1A
#define   ELE11_H          0x1B
#define   EFDPROX_L        0x1C
#define   EFDPROX_H        0x1D
#define   E0BV             0x1E
#define   E1BV             0x1F
#define   E2BV             0x20
#define   E3BV             0x21
#define   E4BV             0x22
#define   E5BV             0x23
#define   E6BV             0x24
#define   E7BV             0x25
#define   E8BV             0x26
#define   E9BV             0x27
#define   E10BV            0x28
#define   E11BV            0x29
#define   EPROXBV          0x2A
#define   MHD_R            0x2B
#define   NHD_R            0x2C
#define   NCL_R            0x2D
#define   FDL_R            0x2E
#define   MHD_F            0x2F
#define   NHD_F            0x30
#define   NCL_F            0x31
#define   FDL_F            0x32
#define   NHD_T            0x33
#define   NCL_T            0x34
#define   FDL_T            0x35
#define   MHDPROXR         0x36
#define   NHDPROXR         0x37
#define   NCLPROXR         0x38
#define   FDLPROXR         0x39
#define   MHDPROXF         0x3A
#define   NHDPROXF         0x3B
#define   NCLPROXF         0x3C
#define   FDLPROXF         0x3D
#define   NHDPROXT         0x3E
#define   NCLPROXT         0x3F
#define   FDLPROXT         0x40
#define   ELE0_T           0x41
#define   ELE0_R           0x42
#define   ELE1_T           0x43
#define   ELE1_R           0x44
#define   ELE2_T           0x45
#define   ELE2_R           0x46
#define   ELE3_T           0x47
#define   ELE3_R           0x48
#define   ELE4_T           0x49
#define   ELE4_R           0x4A
#define   ELE5_T           0x4B
#define   ELE5_R           0x4C
#define   ELE6_T           0x4D
#define   ELE6_R           0x4E
#define   ELE7_T           0x4F
#define   ELE7_R           0x50
#define   ELE8_T           0x51
#define   ELE8_R           0x52
#define   ELE9_T           0x53
#define   ELE9_R           0x54
#define   ELE10_T          0x55
#define   ELE10_R          0x56
#define   ELE11_T          0x57
#define   ELE11_R          0x58
#define   EPROXTTH         0x59
#define   EPROXRTH         0x5A
#define   DEBOUNCE         0x5B
#define   FIL_CFG1         0x5C
#define   FIL_CFG2         0x5D
#define   ELE_CFG          0x5E
#define   CDC0             0x5F
#define   CDC1             0x60
#define   CDC2             0x61
#define   CDC3             0x62
#define   CDC4             0x63
#define   CDC5             0x64
#define   CDC6             0x65
#define   CDC7             0x66
#define   CDC8             0x67
#define   CDC9             0x68
#define   CDC10            0x69
#define   CDC11            0x6A
#define   CDCPROX          0x6B
#define   CDT0_1           0x6C
#define   CDT2_3           0x6D
#define   CDT4_5           0x6E
#define   CDT6_7           0x6F
#define   CDT8_9           0x70
#define   CDT10_11         0x71
#define   CDTPROX          0x72
#define   GPIO_CTRL0       0x73
#define   GPIO_CTRL1       0x74
#define   GPIO_DATA        0x75
#define   GPIO_DIR         0x76
#define   GPIO_EN          0x77
#define   GPIO_SET         0x78
#define   GPIO_CLEAR       0x79
#define   GPIO_TOGGLE      0x7A
#define   AUTO_CFG0        0x7B
#define   AUTO_CFG_CTRL    0x7C
#define   AUTO_CFGU        0x7D
#define   AUTO_CFGL        0x7E
#define   AUTO_CFGT        0x7F
#define   SRST             0x80
#define   TOU_THRESH       0x05
#define   REL_THRESH       0x08

/* command MMA8652 */
#define   CTRL_REG1       0x2A
#define   CTRL_REG2       0x2B
#define   CTRL_REG3       0x2C
#define   CTRL_REG4       0x2D
#define   CTRL_REG5       0x2E
#define   XYZ_DATA        0x0E
#define   OFF_X_REG       0x2F
#define   OFF_Y_REG       0x30
#define   OFF_Z_REG       0x31
#define   WHO_AM_I		  0x0D
#define	  SENSITIVITY_2G  1024

//**************************************************************
//**************************************************************
//**************************************************************
//**************************************************************

/* ZPOZDENI */
#define delay_10us() __delay_cycles(10);
#define MEASURING_T __delay_cycles(70000);
#define MEASURING_RH __delay_cycles(30000);
#define Delay_Reset __delay_cycles(30000);
#define Delay_5ms_BMP180 __delay_cycles(30000);

//**************************************************************************************
//**************************************************************************************
/* Globalni promenne */
unsigned char ACK = 0;
unsigned char j = 0x00;
volatile short int acceldata[6] = {0,0,0,0,0,0};
volatile short int Xout = 0, Yout = 0, Zout = 0;
volatile short int Xout_g = 0, Yout_g = 0, Zout_g = 0;
volatile float XOUTG = 0, YOUTG = 0, ZOUTG = 0;

//**************************************************************************************
//**************************************************************************************

/* Globalni promenne BMP180*/
long int AC1 = 8644;
long int AC2 = -1240;
long int AC3 = -14463;
long int AC4 = -31190;
long int AC5 = 25335;
long int AC6 = 16718;
long int B1 = 6515;
long int B2 = 51;
long int MB = -32768;
long int MC = -11786;	//
long int MD = 2616;

long int UT = 0;
long int oss = 3;
long int UP = 0;
long int X1 = 0;
long int X2 = 0;
long int X21 = 0;
long int XA = 0;
long int B5 = 0;
long int T = 0;

long int B6 = 0;
long int X12 = 0;
long int X22 = 0;
long int X3 = 0;
long int B3 = 0;
long int X13 = 0;
long int X23 = 0;
long int X33 = 0;
long int B4 = 0;
long int B7 = 0;
long int P0 = 0;
long int X141 = 0;
long int X142 = 0;
long int X24 = 0;
long int P = 0;


//***************************************************************************************
//***************************************************************************************
//***************************************************************************************

int main(void)
{
/********** Inicializace **********/

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	InitI2C();
	InitPiezo();
	MPR121_init();
	InitMMA8652();
	//MMA8652_KALIBRACE();

	/*AC1 = RxDBMP180(BMP180W, 0xAA, BMP180R, 0);
	AC2 = RxDBMP180(BMP180W, 0xAC, BMP180R, 0);
	AC3 = RxDBMP180(BMP180W, 0xAE, BMP180R, 0);
	AC4 = RxDBMP180(BMP180W, 0xB0, BMP180R, 0);
	AC5 = RxDBMP180(BMP180W, 0xB2, BMP180R, 0);
	AC6 = RxDBMP180(BMP180W, 0xB4, BMP180R, 0);
	B1 = RxDBMP180(BMP180W, 0xB6, BMP180R, 0);
	B2 = RxDBMP180(BMP180W, 0xB8, BMP180R, 0);
	MB = RxDBMP180(BMP180W, 0xBA, BMP180R, 0);
	//MC = RxDBMP180(BMP180W, 0xBC, BMP180R, 0);
	MD = RxDBMP180(BMP180W, 0xBE, BMP180R, 0);*/


	//__delay_cycles(15000);


/********** Inicializace **********/

	while(1)
	{

		RxDMMA8652(MMA8652W, 0x01, MMA8652R);      // Read data output registers 0x01-0x06
		__delay_cycles(100000);

		Xout = 0;
		Yout = 0;
		Zout = 0;
		Xout_g = 0;
		Yout_g = 0;
		Zout_g = 0;
		XOUTG = 0;
		YOUTG = 0;
		ZOUTG = 0;



		Xout = acceldata[0] << 4;
		Xout += acceldata[1] >> 4;
		Xout &= ~0xF000;
		Yout = acceldata[2] << 4;
		Yout += acceldata[3] >> 4;
		Yout &= ~0xF000;
		Zout = acceldata[4] << 4;
		Zout += acceldata[5] >> 4;
		Zout &= ~0xF000;

		if(0x0800 & Xout)
		{
			Xout_g = -19991;
		if(0x0001 & Xout)
			Xout_g += 10;
		if(0x0002 & Xout)
			Xout_g += 20;
		if(0x0004 & Xout)
			Xout_g += 39;
		if(0x0008 & Xout)
			Xout_g += 78;
		if(0x0010 & Xout)
			Xout_g += 156;
		if(0x0020 & Xout)
			Xout_g += 313;
		if(0x0040 & Xout)
			Xout_g += 625;
		if(0x0080 & Xout)
			Xout_g += 1250;
		if(0x0100 & Xout)
			Xout_g += 2500;
		if(0x0200 & Xout)
			Xout_g += 5000;
		if(0x0400 & Xout)
			Xout_g += 10000;
		XOUTG = Xout_g;
		XOUTG /= 10000;
		}
		else
		{
		if(0x0001 & Xout)
			Xout_g += 10;
		if(0x0002 & Xout)
			Xout_g += 20;
		if(0x0004 & Xout)
			Xout_g += 39;
		if(0x0008 & Xout)
			Xout_g += 78;
		if(0x0010 & Xout)
			Xout_g += 156;
		if(0x0020 & Xout)
			Xout_g += 313;
		if(0x0040 & Xout)
			Xout_g += 625;
		if(0x0080 & Xout)
			Xout_g += 1250;
		if(0x0100 & Xout)
			Xout_g += 2500;
		if(0x0200 & Xout)
			Xout_g += 5000;
		if(0x0400 & Xout)
			Xout_g += 10000;
		XOUTG = Xout_g;
		XOUTG /= 10000;
		}

				if(0x0800 & Yout)
				{
					Yout_g = -19991;
				if(0x0001 & Yout)
					Yout_g += 10;
				if(0x0002 & Yout)
					Yout_g += 20;
				if(0x0004 & Yout)
					Yout_g += 39;
				if(0x0008 & Yout)
					Yout_g += 78;
				if(0x0010 & Yout)
					Yout_g += 156;
				if(0x0020 & Yout)
					Yout_g += 313;
				if(0x0040 & Yout)
					Yout_g += 625;
				if(0x0080 & Yout)
					Yout_g += 1250;
				if(0x0100 & Yout)
					Yout_g += 2500;
				if(0x0200 & Yout)
					Yout_g += 5000;
				if(0x0400 & Yout)
					Yout_g += 10000;
				YOUTG = Yout_g;
				YOUTG /= 10000;
				}
				else
				{
				if(0x0001 & Yout)
					Yout_g += 10;
				if(0x0002 & Yout)
					Yout_g += 20;
				if(0x0004 & Yout)
					Yout_g += 39;
				if(0x0008 & Yout)
					Yout_g += 78;
				if(0x0010 & Yout)
					Yout_g += 156;
				if(0x0020 & Yout)
					Yout_g += 313;
				if(0x0040 & Yout)
					Yout_g += 625;
				if(0x0080 & Yout)
					Yout_g += 1250;
				if(0x0100 & Yout)
					Yout_g += 2500;
				if(0x0200 & Yout)
					Yout_g += 5000;
				if(0x0400 & Yout)
					Yout_g += 10000;
				YOUTG = Yout_g;
				YOUTG /= 10000;
				}

						if(0x0800 & Zout)
						{
							Zout_g = -19991;
						if(0x0001 & Zout)
							Zout_g += 10;
						if(0x0002 & Zout)
							Zout_g += 20;
						if(0x0004 & Zout)
							Zout_g += 39;
						if(0x0008 & Zout)
							Zout_g += 78;
						if(0x0010 & Zout)
							Zout_g += 156;
						if(0x0020 & Zout)
							Zout_g += 313;
						if(0x0040 & Zout)
							Zout_g += 625;
						if(0x0080 & Zout)
							Zout_g += 1250;
						if(0x0100 & Zout)
							Zout_g += 2500;
						if(0x0200 & Zout)
							Zout_g += 5000;
						if(0x0400 & Zout)
							Zout_g += 10000;
						ZOUTG = Zout_g;
						ZOUTG /= 10000;
						}
						else
						{
						if(0x0001 & Zout)
							Zout_g += 10;
						if(0x0002 & Zout)
							Zout_g += 20;
						if(0x0004 & Zout)
							Zout_g += 39;
						if(0x0008 & Zout)
							Zout_g += 78;
						if(0x0010 & Zout)
							Zout_g += 156;
						if(0x0020 & Zout)
							Zout_g += 313;
						if(0x0040 & Zout)
							Zout_g += 625;
						if(0x0080 & Zout)
							Zout_g += 1250;
						if(0x0100 & Zout)
							Zout_g += 2500;
						if(0x0200 & Zout)
							Zout_g += 5000;
						if(0x0400 & Zout)
							Zout_g += 10000;
						ZOUTG = Zout_g;
						ZOUTG /= 10000;
						}


		//Xout_g = Xout / SENSITIVITY_2G;              // Compute X-axis output value in g's
		//Yout_g = Yout / SENSITIVITY_2G;              // Compute Y-axis output value in g's
		//Zout_g = Zout / SENSITIVITY_2G;              // Compute Z-axis output value in g's




		/*TxDBMP180(BMP180W, 0xF4, 0x2E);
		Delay_5ms_BMP180;
		UT = RxDBMP180(BMP180W, 0xF6, BMP180R, 0);
		TxDBMP180(BMP180W, 0xF4, 0xF4);
		Delay_5ms_BMP180;
		UP = RxDBMP180(BMP180W, 0xF6, BMP180R, 1);
		__delay_cycles(1000000);

		X1 = (UT - AC6) * AC5 / 32768;
		X2 =  MC * 2048 / (X1 + MD);
		B5 = X1 + X2;
		T = (B5 + 8) / 16;*/

		/*B6 = B5 - 4000;
		X12 = (B2 * (B6 * B6 / 4096)) / 2048;
		X22 = AC2 * B6 / 2048;
		X3 = X12 + X22;
		B3 = (((AC1 * 4 + X3) << oss) + 2) / 4;
		X13 = AC3 * B6 / 8192;
		X23 = (B1 * (B6 * B6 / 4096)) / 65536;
		X33 = ((X13 + X23) + 2) / 4;
		B4 = (AC4 * (-1)) * (X33 + 32768) / 32768;
		B7 = (UP - B3) * (50000 >> oss);
		if(B7 < 0x80000000)
		{
			P0 = (B7 * 2) / B4;
		}
		else
		{
			P0 = (B7 / B4) * 2;
		}
		X141 = (P0 / 256) * (P0 / 256);
		X142 = (X1 * 3038) / 65536;
		X24 = (-7357 * P0) / 65536;
		P = P0 + (X142 + X24 + 3791) / 16;*/
	}




	/*while(1)
		{
		Start();
		ACK = TxD(Write_MPR121);
						if(ACK == 0)
							goto konec;
						ACK = TxD(ELE_H);
						if(ACK == 0)
		goto konec;
						SDA_1();
						SCL_1();
						delay_10us();
						Start();
						ACK = TxD(Read_MPR121);
												if(ACK == 0)
													goto konec;
		j = 0x00;
		j = RxD(0);
		j &= ~0xF9;
		Stop();
		if(j >= 1)
		{
			PIEZO_1();
			__delay_cycles(150000);
			PIEZO_0();
			//__delay_cycles(150000);
		}
		konec:;*/

		/*
			Start();
				ACK = TxD(SadrHT16W);
				if(ACK == 0)
					goto konec;
				ACK = TxD(SYSSET);
				if(ACK == 0)
					goto konec;

				SDA_0();
				SCL_0();
				delay_10us();
				Stop();

				Start();
				ACK = TxD(SadrHT16W);
				if(ACK == 0)
					goto konec;
				ACK = TxD(ROWSET);
				if(ACK == 0)
					goto konec;

				SDA_0();
				SCL_0();
				delay_10us();
				Stop();

				Start();
						ACK = TxD(SadrHT16W);
						if(ACK == 0)
							goto konec;
						ACK = TxD(DUTYSET);
						if(ACK == 0)
							goto konec;

						SDA_0();
						SCL_0();
						delay_10us();
						Stop();

				Start();
				ACK = TxD(SadrHT16W);
				if(ACK == 0)
					goto konec;
				ACK = TxD(DISSET);
				if(ACK == 0)
					goto konec;

				SDA_0();
				SCL_0();
				delay_10us();
				Stop();

		while(1)
		{


			Start();
					ACK = TxD(SadrHT16W);
					if(ACK == 0)
						goto konec;
					ACK = TxD(0x00);
					if(ACK == 0)
						goto konec;

					ACK = TxD(0x36);
					if(ACK == 0)
					{
					goto konec;
					}
					ACK = TxD(0x28);
					if(ACK == 0)
					{
						goto konec;
					}
									SDA_0();
									SCL_0();
									delay_10us();

									Stop();
									delay_10us();


									Start();
									ACK = TxD(SadrHT16W);
									if(ACK == 0)
									{
										goto konec;
									}
									ACK = TxD(0x81);
									if(ACK == 0)
									{
										goto konec;
									}

									SDA_0();
									SCL_0();
									delay_10us();
									Stop();

									konec:;

									__delay_cycles(1000000);
		}
		*/
	//return 0;

}
//***************************************************************************************
//Init I2C Vystupni SDA i SCL V Log.1
void InitI2C(void)
{
	P1DIR |= 0x0C;	//SDA,SCL pin vystupni
	P1OUT |= 0x0C;	//SDA,SCL Log. 1

}
//***************************************************************************************
//Init Piezo Log.0
void InitPiezo(void)
{
	P3DIR |= 0x10;	//Piezo vystupni
	PIEZO_0();		//Piezo Log.0

}
//***************************************************************************************
//Init MMA8652FC
void InitMMA8652(void)
{
	TxDBMP180(MMA8652W, CTRL_REG2, 0x40);		//		Reset all registers to POR values
	__delay_cycles(10000);						//		10ms delay
	TxDBMP180(MMA8652W, XYZ_DATA, 0x00);		//		+/-2g range with ~0.977mg/LSB
	TxDBMP180(MMA8652W, CTRL_REG2, 0x02);		//		High Resolution mode
	TxDBMP180(MMA8652W, CTRL_REG3, 0x00);		//		Push-pull, active low interrupt
	TxDBMP180(MMA8652W, CTRL_REG4, 0x01);		//		Enable DRDY interrupt
	TxDBMP180(MMA8652W, CTRL_REG5, 0x01);		//		DRDY interrupt routed to INT1 - PTA5
	TxDBMP180(MMA8652W, CTRL_REG1, 0x39);		//		ODR = 1.56Hz, Active mode
}
//***************************************************************************************
//Kalibrace Akcelerometru
void MMA8652_KALIBRACE(void)
{
volatile short int X_offset = 0, Y_offset = 0, Z_offset = 0;
volatile short int Xout_12_bit = 0, Yout_12_bit = 0, Zout_12_bit = 0;

	 TxDBMP180(MMA8652W, CTRL_REG1, 0x00);		// Standby mode
	 RxDMMA8652(MMA8652W, 0x01, MMA8652R);      // Read data output registers 0x01-0x06
	 __delay_cycles(300000);
	 RxDMMA8652(MMA8652W, 0x01, MMA8652R);      // Read data output registers 0x01-0x06
	 __delay_cycles(300000);
	 RxDMMA8652(MMA8652W, 0x01, MMA8652R);      // Read data output registers 0x01-0x06
	 __delay_cycles(300000);

	 Xout_12_bit = acceldata[0] << 4;
	 Xout_12_bit += acceldata[1] >> 4;
	 Xout_12_bit &= ~0xF000;
	 Yout_12_bit = acceldata[2] << 4;
	 Yout_12_bit += acceldata[3] >> 4;
	 Yout_12_bit &= ~0xF000;
	 Zout_12_bit = acceldata[4] << 4;
	 Zout_12_bit += acceldata[5] >> 4;
	 Zout_12_bit &= ~0xF000;

	 X_offset = Xout_12_bit / 2 * (-1);          // Compute X-axis offset correction value
	 Y_offset = Yout_12_bit / 2 * (-1);          // Compute Y-axis offset correction value
	 Z_offset = (Zout_12_bit - SENSITIVITY_2G) / 2 * (-1);         // Compute Z-axis offset correction value

	 TxDOFFMMA8652(MMA8652W, OFF_X_REG, X_offset);
	 TxDOFFMMA8652(MMA8652W, OFF_Y_REG, Y_offset);
	 TxDOFFMMA8652(MMA8652W, OFF_Z_REG, Z_offset);

	 TxDBMP180(MMA8652W, CTRL_REG1, 0x39);          // Active mode again

}
//***************************************************************************************
//Zapis do Slave BMP180
void TxDOFFMMA8652(unsigned char addrw, unsigned char reg, short int value)
{
	unsigned char i;
	volatile unsigned char k = 0;

	Start();
	for(i = 0; i < 8; i++, addrw <<= 1)
		{
			if(addrw & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		if(k == 0)
			k = 0;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, reg <<= 1)
	{
		if(reg & 0x80)
		{
			SDA_1();
		}
		else
		{
			SDA_0();
		}
		ClockI2C();
	}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, value <<= 1)
		{
			if(value & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
	SCL_1();
	delay_10us();
	k = !SDA_IN;
	SCL_0();
	SDA_OUTPUT;
	SDA_0();
	delay_10us();
	k = 0;
	Stop();

}
//***************************************************************************************
//Cteni z Slave BMP
void RxDMMA8652(unsigned char addrw, unsigned char reg, unsigned char addrr)
{
	unsigned char i, j;
	volatile unsigned char k;

	Start();
		for(i = 0; i < 8; i++, addrw <<= 1)
			{
				if(addrw & 0x80)
				{
					SDA_1();
				}
				else
				{
					SDA_0();
				}
				ClockI2C();
			}
		SDA_INPUT;
			SCL_1();
			delay_10us();
			k = !SDA_IN;
			if(k == 0)
				k = 0;
			SCL_0();
			SDA_OUTPUT;
			SDA_0();
			delay_10us();
			k = 0;
		for(i = 0; i < 8; i++, reg <<= 1)
		{
			if(reg & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
		SDA_INPUT;
			SCL_1();
			delay_10us();
			k = !SDA_IN;
			if(k == 0)
				k = 0;
			SCL_0();
			SDA_OUTPUT;
			SDA_0();
			delay_10us();
			k = 0;
	Restart();


	for(i = 0; i < 8; i++, addrr <<= 1)
			{
				if(addrr & 0x80)
				{
					SDA_1();
				}
				else
				{
					SDA_0();
				}
				ClockI2C();
			}
			SDA_INPUT;
				SCL_1();
				delay_10us();
				k = !SDA_IN;
				if(k == 0)
					k = 0;
				SCL_0();
				SDA_OUTPUT;
				SDA_0();
				delay_10us();
				k = 0;

	for(j = 0; j < 6; j++)
	{
		SDA_INPUT;
		for(i = 0; i < 8; i++)
		{
			acceldata[j] <<= 1;
			SCL_1();
			delay_10us();
			if(SDA_IN)
			{
				acceldata[j] |= 0x1;
			}
			SCL_0();
			delay_10us();
		}
		acceldata[j] &= ~0xFF00;
		if(j != 5)
		{
		SDA_OUTPUT;	// ACK
		SDA_0();
		ClockI2C();
		}
	}
	SDA_OUTPUT;	// NACK
	SDA_1();
	ClockI2C();
	SDA_0();
	Stop();
}
//***************************************************************************************
//Sekvence S
void Start(void)
{
	SDA_0();
	delay_10us();
	SCL_0();
	delay_10us();
}
//***************************************************************************************
//Sekvence P
void Stop(void)
{
	delay_10us();
	SCL_1();
	delay_10us();
	SDA_1();
	delay_10us();
}
//***************************************************************************************
//Sekvence Restart
void Restart(void)
{
	SDA_1();
	SCL_1();
	delay_10us();
	Start();
}


//***************************************************************************************
//Sekvence Clock I2C
void ClockI2C(void)
{
	delay_10us();
	SCL_1();
	delay_10us();
	SCL_0();
	delay_10us();
}
//***************************************************************************************
//Soft Reset
void SReset(void)
{
	Start();
	ACK = TxD(SadrSHTW);
	if(ACK == 0)
	{
		goto konec2;
	}
	ACK = TxD(SOFT_RESET);
	if(ACK == 0)
	{
		goto konec2;
	}
	Stop();
	Delay_Reset;
	konec2:;
}
//***************************************************************************************
//Zapis do Slave
unsigned char TxD(unsigned char addr)
{
	unsigned char i;
	unsigned char b;
	//unsigned char a = _a;
	for(i = 0; i < 8; i++, addr <<= 1)
	{
		if(addr & 0x80)
		{
			SDA_1();
		}
		else
		{
			SDA_0();
		}
		ClockI2C();
	}
	SDA_INPUT;
	SCL_1();
	delay_10us();
	b = !SDA_IN;
	SCL_0();
	SDA_OUTPUT;
	SDA_0();
	delay_10us();

	return b;
}
//***************************************************************************************
//Cteni z Slave
unsigned char RxD(unsigned char send_ack)
{
	unsigned char i;
	unsigned char c = 0x00;
	SDA_INPUT;

	for(i = 0; i < 8; i++)
	{
		c <<= 1;
		SCL_1();
		delay_10us();
		if(SDA_IN)
		{
			c |= 0x1;
		}
		SCL_0();
		delay_10us();
	}

	if(send_ack)
	{
		SDA_OUTPUT;	// ACK
		SDA_0();
	}
	else
	{
		SDA_OUTPUT;
		SDA_1();	// NACK
	}

	SCL_1();
	delay_10us();
	SCL_0();
	SDA_0();
	delay_10us();

	return c;
}

//***************************************************************************************
//Cteni z Slave BMP
long int RxDBMP180(unsigned char addrw, unsigned char reg, unsigned char addrr, unsigned char tlak)
{
	unsigned char i;
	long int value1, value2, value3;
	long int vysledek;
	volatile unsigned char k;

	Start();
		for(i = 0; i < 8; i++, addrw <<= 1)
			{
				if(addrw & 0x80)
				{
					SDA_1();
				}
				else
				{
					SDA_0();
				}
				ClockI2C();
			}
		SDA_INPUT;
			SCL_1();
			delay_10us();
			k = !SDA_IN;
			if(k == 0)
				k = 0;
			SCL_0();
			SDA_OUTPUT;
			SDA_0();
			delay_10us();
			k = 0;
		for(i = 0; i < 8; i++, reg <<= 1)
		{
			if(reg & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
		SDA_INPUT;
			SCL_1();
			delay_10us();
			k = !SDA_IN;
			if(k == 0)
				k = 0;
			SCL_0();
			SDA_OUTPUT;
			SDA_0();
			delay_10us();
			k = 0;
	Restart();


	for(i = 0; i < 8; i++, addrr <<= 1)
			{
				if(addrr & 0x80)
				{
					SDA_1();
				}
				else
				{
					SDA_0();
				}
				ClockI2C();
			}
			SDA_INPUT;
				SCL_1();
				delay_10us();
				k = !SDA_IN;
				if(k == 0)
					k = 0;
				SCL_0();
				SDA_OUTPUT;
				SDA_0();
				delay_10us();
				k = 0;
	SDA_INPUT;
	value1 = 0;
	for(i = 0; i < 8; i++)
	{
		value1 <<= 1;
		SCL_1();
		delay_10us();
		if(SDA_IN)
		{
			value1 |= 0x1;
		}
		SCL_0();
		delay_10us();
	}

	SDA_OUTPUT;	// ACK
	SDA_0();
	SCL_1();
	delay_10us();
	SCL_0();
	SDA_0();
	delay_10us();

	SDA_INPUT;
	value2 = 0;
		for(i = 0; i < 8; i++)
		{
			value2 <<= 1;
			SCL_1();
			delay_10us();
			if(SDA_IN)
			{
				value2 |= 0x1;
			}
			SCL_0();
			delay_10us();
		}

		SDA_OUTPUT;	// ACK
		SDA_1();
		SCL_1();
		delay_10us();
		SCL_0();
		SDA_0();
		delay_10us();

		if(tlak == 1)
		{
		SDA_INPUT;
			value3 = 0;
				for(i = 0; i < 8; i++)
				{
					value3 <<= 1;
					SCL_1();
					delay_10us();
					if(SDA_IN)
					{
						value3 |= 0x1;
					}
					SCL_0();
					delay_10us();
				}

				SDA_OUTPUT;	// ACK
				SDA_1();
				SCL_1();
				delay_10us();
				SCL_0();
				SDA_0();
				delay_10us();
		}
		Stop();
		vysledek = 0;
		if(tlak == 0)
		{
		vysledek = value1<<8;
		vysledek += value2;
		}

		if(tlak == 1)
		{
			vysledek = value1<<16;
			vysledek += value2<<8;
			vysledek += value3;
			vysledek = vysledek>>5;
		}
	return vysledek;
}
//***************************************************************************************
//Zapis do Slave BMP180
void TxDBMP180(unsigned char addrw, unsigned char reg, unsigned char value)
{
	unsigned char i;
	volatile unsigned char k = 0;

	Start();
	for(i = 0; i < 8; i++, addrw <<= 1)
		{
			if(addrw & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		if(k == 0)
			k = 0;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, reg <<= 1)
	{
		if(reg & 0x80)
		{
			SDA_1();
		}
		else
		{
			SDA_0();
		}
		ClockI2C();
	}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, value <<= 1)
		{
			if(value & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
	SCL_1();
	delay_10us();
	k = !SDA_IN;
	SCL_0();
	SDA_OUTPUT;
	SDA_0();
	delay_10us();
	k = 0;
	Stop();

}
//***************************************************************************************
//Zapis do Slave MPR121
void TxDMPR121(unsigned char addr, unsigned char value)
{
	volatile unsigned char i;
	volatile unsigned char k = 0;
	volatile unsigned char writempr121;
	writempr121 = Write_MPR121;
	Start();
	for(i = 0; i < 8; i++, writempr121 <<= 1)
		{
			if(writempr121 & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		if(k == 0)
			k = 0;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, addr <<= 1)
	{
		if(addr & 0x80)
		{
			SDA_1();
		}
		else
		{
			SDA_0();
		}
		ClockI2C();
	}
	SDA_INPUT;
		SCL_1();
		delay_10us();
		k = !SDA_IN;
		SCL_0();
		SDA_OUTPUT;
		SDA_0();
		delay_10us();
		k = 0;
	for(i = 0; i < 8; i++, value <<= 1)
		{
			if(value & 0x80)
			{
				SDA_1();
			}
			else
			{
				SDA_0();
			}
			ClockI2C();
		}
	SDA_INPUT;
	SCL_1();
	delay_10us();
	k = !SDA_IN;
	SCL_0();
	SDA_OUTPUT;
	SDA_0();
	delay_10us();
	k = 0;
	Stop();

}
//***************************************************************************************
//Inicializace MPR121
void MPR121_init()
{
	TxDMPR121(ELE_CFG, 0x00);			//0x5E
	TxDMPR121(MHD_R, 0x01);				//0x2B
	TxDMPR121(NHD_R, 0x01);				//0x2C
	TxDMPR121(NCL_R, 0x00);				//0x2D
	TxDMPR121(FDL_R, 0x00);				//0x2E
	TxDMPR121(MHD_F, 0x01);				//0x2F
	TxDMPR121(NHD_F, 0x01);				//0x30
	TxDMPR121(NCL_F, 0xFF);				//0x31
	TxDMPR121(FDL_F, 0x00);				//0x32
	TxDMPR121(ELE0_T, TOU_THRESH);		//0x41
	TxDMPR121(ELE0_R, REL_THRESH);		//0x42
	TxDMPR121(ELE1_T, TOU_THRESH);		//0x43
	TxDMPR121(ELE1_R, REL_THRESH);		//0x44
	TxDMPR121(ELE2_T, TOU_THRESH);		//0x45
	TxDMPR121(ELE2_R, REL_THRESH);		//0x46
	TxDMPR121(ELE3_T, TOU_THRESH);		//0x47
	TxDMPR121(ELE3_R, REL_THRESH);		//0x48
	TxDMPR121(ELE4_T, TOU_THRESH);		//0x49
	TxDMPR121(ELE4_R, REL_THRESH);		//0x4A
	TxDMPR121(ELE5_T, TOU_THRESH);		//0x4B
	TxDMPR121(ELE5_R, REL_THRESH);		//0x4C
	TxDMPR121(ELE6_T, TOU_THRESH);		//0x4D
	TxDMPR121(ELE6_R, REL_THRESH);		//0x4E
	TxDMPR121(ELE7_T, TOU_THRESH);		//0x4F
	TxDMPR121(ELE7_R, REL_THRESH);		//0x50
	TxDMPR121(ELE9_T, TOU_THRESH);
	TxDMPR121(ELE9_R, REL_THRESH);
	TxDMPR121(ELE10_T, TOU_THRESH);
	TxDMPR121(ELE10_R, REL_THRESH);

	TxDMPR121(AUTO_CFG0, 0x0B);			//0x7B
	TxDMPR121(AUTO_CFGU, 0xC9);			//0x7D
	TxDMPR121(AUTO_CFGL, 0x82);			//0x7E
	TxDMPR121(AUTO_CFGT, 0xB5);			//0x7F
	TxDMPR121(FIL_CFG2, 0x04);			//0x5D
	TxDMPR121(ELE_CFG, 0x0B);			//0x5E

	/*TxDMPR121(GPIO_CTRL0, 0x00);
	TxDMPR121(GPIO_CTRL1, 0x00);
	TxDMPR121(GPIO_DIR, 0x00);
	TxDMPR121(GPIO_EN, 0x00);




	TxDMPR121(FIL_CFG1, 0x0B);
	*/
}

//#define   TOU_THRESH       0x08
//#define   REL_THRESH       0x05
